from __future__ import annotations
from io import TextIOWrapper

from Corpus.Sentence import Sentence
from DependencyParser.ParserEvaluationScore import ParserEvaluationScore
from FrameNet.FrameNet import FrameNet
from MorphologicalAnalysis.FsmMorphologicalAnalyzer import FsmMorphologicalAnalyzer
from PropBank.FramesetList import FramesetList
from WordNet.WordNet import WordNet

from AnnotatedSentence.AnnotatedPhrase import AnnotatedPhrase
from AnnotatedSentence.AnnotatedWord import AnnotatedWord


class AnnotatedSentence(Sentence):

    __file_name: str

    def __init__(self,
                 fileOrStr=None,
                 fileName=None):
        """
        Converts a simple sentence to an annotated sentence

        PARAMETERS
        ----------
        fileOrStr
            Simple sentence
        """
        self.words = []
        word_array = []
        if fileOrStr is not None:
            if fileName is not None:
                self.__file_name = fileName
            if isinstance(fileOrStr, TextIOWrapper):
                line = fileOrStr.readline()
                fileOrStr.close()
                word_array = line.rstrip().split(" ")
            elif isinstance(fileOrStr, str):
                word_array = fileOrStr.split(" ")
            for word in word_array:
                if len(word) > 0:
                    self.words.append(AnnotatedWord(word))

    def getShallowParseGroups(self) -> list:
        """
        The method constructs all possible shallow parse groups of a sentence.

        RETURNS
        -------
        list
            Shallow parse groups of a sentence.
        """
        shallow_parse_groups = []
        previous_word = None
        current = None
        for i in range(self.wordCount()):
            word = self.getWord(i)
            if isinstance(word, AnnotatedWord):
                if previous_word is None:
                    current = AnnotatedPhrase(i, word.getShallowParse())
                else:
                    if isinstance(previous_word, AnnotatedWord) and previous_word.getShallowParse() is not None \
                            and previous_word.getShallowParse() != word.getShallowParse():
                        shallow_parse_groups.append(current)
                        current = AnnotatedPhrase(i, word.getShallowParse())
                current.addWord(word)
                previous_word = word
        shallow_parse_groups.append(current)
        return shallow_parse_groups

    def containsPredicate(self) -> bool:
        """
        The method checks all words in the sentence and returns true if at least one of the words is annotated with
        PREDICATE tag.

        RETURNS
        -------
        bool
            True if at least one of the words is annotated with PREDICATE tag; False otherwise.
        """
        for word in self.words:
            if isinstance(word, AnnotatedWord):
                if word.getArgumentList() is not None:
                    if word.getArgumentList().containsPredicate():
                        return True
        return False

    def containsFramePredicate(self) -> bool:
        """
        The method checks all words in the sentence and returns true if at least one of the words is annotated with
        PREDICATE tag.

        RETURNS
        -------
        bool
            True if at least one of the words is annotated with PREDICATE tag; False otherwise.
        """
        for word in self.words:
            if isinstance(word, AnnotatedWord):
                if word.getFrameElementList() is not None:
                    if word.getFrameElementList().containsPredicate():
                        return True
        return False

    def updateConnectedPredicate(self,
                                 previousId: str,
                                 currentId: str) -> bool:
        """
        Replaces id's of predicates, which have previousId as synset id, with currentId. Replaces also predicate id's of
        frame elements, which have predicate id's previousId, with currentId.
        :param previousId: Previous id of the synset.
        :param currentId: Replacement id.
        :return: Returns true, if any replacement has been done; false otherwise.
        """
        modified = False
        for word in self.words:
            if isinstance(word, AnnotatedWord):
                argument_list = word.getArgumentList()
                if argument_list is not None:
                    if argument_list.containsPredicateWithId(previousId):
                        argument_list.updateConnectedId(previousId, currentId)
                        modified = True
                frame_element_list = word.getFrameElementList()
                if frame_element_list is not None:
                    if frame_element_list.containsPredicateWithId(previousId):
                        frame_element_list.updateConnectedId(previousId, currentId)
                        modified = True
        return modified

    def predicateCandidates(self, framesetList: FramesetList) -> list:
        """
        The method returns all possible words, which is
        1. Verb
        2. Its semantic tag is assigned
        3. A frameset exists for that semantic tag

        PARAMETERS
        ----------
        framesetList : FramesetList
            Frameset list that contains all frames for Turkish

        RETURNS
        -------
        A list of words, which are verbs, semantic tags assigned, and framesetlist assigned for that tag.
        """
        candidate_list = []
        for word in self.words:
            if isinstance(word, AnnotatedWord):
                if word.getParse() is not None and word.getParse().isVerb() and word.getSemantic() is not None \
                        and framesetList.frameExists(word.getSemantic()):
                    candidate_list.append(word)
        for i in range(2):
            for j in range(len(self.words) - i - 1):
                annotated_word = self.words[j]
                next_annotated_word = self.words[j + 1]
                if isinstance(annotated_word, AnnotatedWord) and isinstance(next_annotated_word, AnnotatedWord):
                    if annotated_word not in candidate_list and next_annotated_word in candidate_list \
                            and annotated_word.getSemantic() is not None \
                            and annotated_word.getSemantic() == next_annotated_word.getSemantic():
                        candidate_list.append(annotated_word)
        return candidate_list

    def predicateFrameCandidates(self, frameNet: FrameNet) -> list:
        """
        The method returns all possible words, which is
        1. Verb
        2. Its semantic tag is assigned
        3. A lexicalUnit exists for that semantic tag

        PARAMETERS
        ----------
        frameNet : FrameNet
            FrameNet that contains all frames for Turkish

        RETURNS
        -------
        A list of words, which are verbs, semantic tags assigned, and frame assigned for that tag.
        """
        candidate_list = []
        for word in self.words:
            if isinstance(word, AnnotatedWord):
                if word.getParse() is not None and word.getParse().isVerb() and word.getSemantic() is not None \
                        and frameNet.lexicalUnitExists(word.getSemantic()):
                    candidate_list.append(word)
        for i in range(2):
            for j in range(len(self.words) - i - 1):
                annotated_word = self.words[j]
                next_annotated_word = self.words[j + 1]
                if isinstance(annotated_word, AnnotatedWord) and isinstance(next_annotated_word, AnnotatedWord):
                    if annotated_word not in candidate_list and next_annotated_word in candidate_list \
                            and annotated_word.getSemantic() is not None \
                            and annotated_word.getSemantic() == next_annotated_word.getSemantic():
                        candidate_list.append(annotated_word)
        return candidate_list

    def getPredicate(self, index: int) -> str:
        """
        Returns the nearest predicate to the index'th word in the sentence.

        PARAMETERS
        ----------
        index : int
            Word index

        RETURNS
        -------
        str
            The nearest predicate to the index'th word in the sentence.
        """
        count1 = 0
        count2 = 0
        data = ""
        word = []
        parse = []
        if index < self.wordCount():
            for i in range(self.wordCount()):
                word.append(self.getWord(i))
                parse.append(self.getWord(i).getParse())
            for i in range(index, -1, -1):
                if parse[i] is not None and parse[i].getRootPos() is not None and parse[i].getPos() is not None \
                        and parse[i].getRootPos() == "VERB" and parse[i].getPos() == "VERB":
                    count1 = index - i
                    break
            for i in range(index, self.wordCount() - index):
                if parse[i] is not None and parse[i].getRootPos() is not None and parse[i].getPos() is not None \
                        and parse[i].getRootPos() == "VERB" and parse[i].getPos() == "VERB":
                    count2 = i - index
                    break
            if count1 > count2:
                data = word[count1].getName()
            else:
                data = word[count2].getName()
        return data

    def getFileName(self) -> str:
        """
        Returns file name of the sentence

        RETURNS
        -------
        str
            File name of the sentence
        """
        return self.__file_name

    def removeWord(self, index: int):
        """
        Removes the i'th word from the sentence

        PARAMETERS
        ----------
        index : int
            Word index
        """
        self.words.pop(index)

    def toStems(self) -> str:
        """
        The toStems method returns an accumulated string of each word's stems in words {@link ArrayList}.
        If the parse of the word does not exist, the method adds the surfaceform to the resulting string.

        RETURNS
        -------
        str
             String result which has all the stems of each item in words {@link ArrayList}.
        """
        if len(self.words) > 0:
            annotated_word = self.words[0]
            if annotated_word.getParse() is not None:
                result = annotated_word.getParse().getWord().getName()
            else:
                result = annotated_word.getName()
            for i in range(1, len(self.words)):
                annotated_word = self.words[i]
                if annotated_word.getParse() is not None:
                    result = result + " " + annotated_word.getParse().getWord().getName()
                else:
                    result = result + " " + annotated_word.getName()
            return result
        else:
            return ""

    def compareParses(self, sentence: AnnotatedSentence) -> ParserEvaluationScore:
        """
        Compares the sentence with the given sentence and returns a parser evaluation score for this comparison. The result
        is calculated by summing up the parser evaluation scores of word by word dpendency relation comparisons.
        :param sentence: Sentence to be compared.
        :return: A parser evaluation score object.
        """
        score = ParserEvaluationScore()
        for i in range(self.wordCount()):
            relation1 = self.words[i].getUniversalDependency()
            relation2 = sentence.getWord(i).getUniversalDependency()
            if relation1 is not None and relation2 is not None:
                score.add(relation1.compareRelations(relation2))
        return score

    def save(self):
        """
        Saves the current sentence.
        """
        self.writeToFile(self.__file_name)

    def getUniversalDependencyFormat(self, path: str = None) -> str:
        """
        Returns the connlu format of the sentence with appended prefix string based on the path.
        :param path: Path of the sentence.
        :return: The connlu format of the sentence with appended prefix string based on the path.
        """
        if path is None:
            result = "# sent_id = " + self.getFileName() + "\n" + "# text = " + self.toString() + "\n"
        else:
            result = "# sent_id = " + path + self.getFileName() + "\n" + "# text = " + self.toString() + "\n"
        for i in range(self.wordCount()):
            word = self.getWord(i)
            if isinstance(word, AnnotatedWord):
                result += str(i + 1) + "\t" + word.getUniversalDependencyFormat(self.wordCount()) + "\n"
        result += "\n"
        return result

    def constructLiterals(self, wordNet: WordNet, fsm: FsmMorphologicalAnalyzer, wordIndex: int) -> list:
        """
        Creates a list of literal candidates for the i'th word in the sentence. It combines the results of
        1. All possible root forms of the i'th word in the sentence
        2. All possible 2-word expressions containing the i'th word in the sentence
        3. All possible 3-word expressions containing the i'th word in the sentence

        PARAMETERS
        ----------
        wordNet : WordNet
            Turkish wordnet
        fsm : FsmMorphologicalAnalyzer
            Turkish morphological analyzer
        wordIndex : int
            Word index

        RETURNS
        -------
        list
            List of literal candidates containing all possible root forms and multiword expressions.
        """
        word = self.getWord(wordIndex)
        possible_literals = []
        if isinstance(word, AnnotatedWord):
            morphological_parse = word.getParse()
            metamorphic_parse = word.getMetamorphicParse()
            possible_literals.extend(wordNet.constructLiterals(morphological_parse.getWord().getName(),
                                                               morphological_parse,
                                                               metamorphic_parse,
                                                               fsm))
            first_succeeding_word = None
            second_succeeding_word = None
            if self.wordCount() > wordIndex + 1:
                first_succeeding_word = self.getWord(wordIndex + 1)
                if self.wordCount() > wordIndex + 2:
                    second_succeeding_word = self.getWord(wordIndex + 2)
            if first_succeeding_word is not None and isinstance(first_succeeding_word, AnnotatedWord):
                if second_succeeding_word is not None and isinstance(second_succeeding_word, AnnotatedWord):
                    possible_literals.extend(wordNet.constructIdiomLiterals(fsm,
                                                                            word.getParse(),
                                                                            word.getMetamorphicParse(),
                                                                            first_succeeding_word.getParse(),
                                                                            first_succeeding_word.getMetamorphicParse(),
                                                                            second_succeeding_word.getParse(),
                                                                            second_succeeding_word.getMetamorphicParse()))
                possible_literals.extend(wordNet.constructIdiomLiterals(fsm,
                                                                        word.getParse(),
                                                                        word.getMetamorphicParse(),
                                                                        first_succeeding_word.getParse(),
                                                                        first_succeeding_word.getMetamorphicParse()))
        return possible_literals

    def constructSynSets(self,
                         wordNet: WordNet,
                         fsm: FsmMorphologicalAnalyzer,
                         wordIndex: int) -> list:
        """
        Creates a list of synset candidates for the i'th word in the sentence. It combines the results of
        1. All possible synsets containing the i'th word in the sentence
        2. All possible synsets containing 2-word expressions, which contains the i'th word in the sentence
        3. All possible synsets containing 3-word expressions, which contains the i'th word in the sentence

        PARAMETERS
        ----------
        wordNet : WordNet
            Turkish wordnet
        fsm : FsmMorphologicalAnalyzer
            Turkish morphological analyzer
        wordIndex : int
            Word index

        RETURNS
        -------
        list
            List of synset candidates containing all possible root forms and multiword expressions.
        """
        word = self.getWord(wordIndex)
        possible_syn_sets = []
        if isinstance(word, AnnotatedWord):
            morphological_parse = word.getParse()
            metamorphic_parse = word.getMetamorphicParse()
            possible_syn_sets.extend(wordNet.constructSynSets(morphological_parse.getWord().getName(),
                                                              morphological_parse,
                                                              metamorphic_parse,
                                                              fsm))
            first_preceding_word = None
            second_preceding_word = None
            first_succeeding_word = None
            second_succeeding_word = None
            if wordIndex > 0:
                first_preceding_word = self.getWord(wordIndex - 1)
                if wordIndex > 1:
                    second_preceding_word = self.getWord(wordIndex - 2)
            if self.wordCount() > wordIndex + 1:
                first_succeeding_word = self.getWord(wordIndex + 1)
                if self.wordCount() > wordIndex + 2:
                    second_succeeding_word = self.getWord(wordIndex + 2)
            if first_preceding_word is not None and isinstance(first_preceding_word, AnnotatedWord):
                if second_preceding_word is not None and isinstance(second_preceding_word, AnnotatedWord):
                    possible_syn_sets.extend(wordNet.constructIdiomSynSets(fsm, second_preceding_word.getParse(),
                                                                         second_preceding_word.getMetamorphicParse(),
                                                                         first_preceding_word.getParse(),
                                                                         first_preceding_word.getMetamorphicParse(),
                                                                         word.getParse(), word.getMetamorphicParse()))
                possible_syn_sets.extend(wordNet.constructIdiomSynSets(fsm, first_preceding_word.getParse(),
                                                                     first_preceding_word.getMetamorphicParse(),
                                                                     word.getParse(), word.getMetamorphicParse()))
            if first_succeeding_word is not None and isinstance(first_succeeding_word, AnnotatedWord):
                if second_succeeding_word is not None and isinstance(second_succeeding_word, AnnotatedWord):
                    possible_syn_sets.extend(wordNet.constructIdiomSynSets(fsm, word.getParse(),
                                                                         word.getMetamorphicParse(),
                                                                         first_succeeding_word.getParse(),
                                                                         first_succeeding_word.getMetamorphicParse(),
                                                                         second_succeeding_word.getParse(),
                                                                         second_succeeding_word.getMetamorphicParse()))
                possible_syn_sets.extend(wordNet.constructIdiomSynSets(fsm, word.getParse(), word.getMetamorphicParse(),
                                                                     first_succeeding_word.getParse(),
                                                                     first_succeeding_word.getMetamorphicParse()))
        return possible_syn_sets



from Corpus.Sentence import Sentence


class AnnotatedPhrase(Sentence):

    __word_index: int
    __tag: str

    def __init__(self,
                 wordIndex: int,
                 tag: str):
        """
        Constructor for AnnotatedPhrase. AnnotatedPhrase stores information about phrases such as
        Shallow Parse phrases or named entity phrases.

        PARAMETERS
        ----------
        wordIndex : int
            Starting index of the first word in the phrase w.r.t. original sentence the phrase occurs.
        tag : str
            Tag of the phrase. Corresponds to the shallow parse or named entity tag.
        """
        super().__init__()
        self.__word_index = wordIndex
        self.__tag = tag

    def getWordIndex(self) -> int:
        """
        Accessor for the wordIndex attribute.

        RETURNS
        -------
        int
            Starting index of the first word in the phrase w.r.t. original sentence the phrase occurs.
        """
        return self.__word_index

    def getTag(self) -> str:
        """
        Accessor for the tag attribute.

        RETURNS
        -------
        str
            Tag of the phrase. Corresponds to the shallow parse or named entity tag.
        """
        return self.__tag



from Corpus.WordFormat import WordFormat
from DependencyParser.Universal.UniversalDependencyRelation import UniversalDependencyRelation
from Dictionary.Word import Word
from FrameNet.FrameElementList import FrameElementList
from MorphologicalAnalysis.FsmParse import FsmParse
from MorphologicalAnalysis.MetamorphicParse import MetamorphicParse
from MorphologicalAnalysis.MorphologicalParse import MorphologicalParse
from MorphologicalAnalysis.MorphologicalTag import MorphologicalTag
from NamedEntityRecognition.Gazetteer import Gazetteer
from NamedEntityRecognition.NamedEntityType import NamedEntityType
from NamedEntityRecognition.Slot import Slot
from PropBank.Argument import Argument
from PropBank.ArgumentList import ArgumentList
from SentiNet.PolarityType import PolarityType

import re

from AnnotatedSentence.Language import Language
from AnnotatedSentence.ViewLayerType import ViewLayerType


class AnnotatedWord(Word):
    """
     * In order to add another layer, do the following:
     * 1. Select a name for the layer.
     * 2. Add a new constant to ViewLayerType.
     * 3. Add private attribute.
     * 4. Add an if-else to the constructor, where you set the private attribute with the layer name.
     * 5. Update toString method.
     * 6. Add initial value to the private attribute in other constructors.
     * 7. Update getLayerInfo.
     * 8. Add getter and setter methods.
    """
    __parse: MorphologicalParse
    __metamorphic_parse: MetamorphicParse
    __semantic: str
    __named_entity_type: NamedEntityType
    __argument_list: ArgumentList
    __frame_element_list: FrameElementList
    __shallow_parse: str
    __universal_dependency: UniversalDependencyRelation
    __slot: Slot
    __polarity: PolarityType
    __ccg: str
    __pos_tag: str
    __language: Language

    def __init__(self,
                 word: str,
                 layerType=None):
        """
        Constructor for the AnnotatedWord class. Gets the word with its annotation layers as input and sets the
        corresponding layers.

        PARAMETERS
        ----------
        word : str
            Input word with annotation layers
        """
        self.__parse = None
        self.__metamorphic_parse = None
        self.__semantic = None
        self.__named_entity_type = None
        self.__argument_list = None
        self.__frame_element_list = None
        self.__shallow_parse = None
        self.__universal_dependency = None
        self.__slot = None
        self.__polarity = None
        self.__ccg = None
        self.__pos_tag = None
        self.__language = Language.TURKISH
        if layerType is None:
            split_layers = re.compile("[{}]").split(word)
            for layer in split_layers:
                if len(layer) == 0:
                    continue
                if "=" not in layer:
                    self.name = layer
                    continue
                layerType = layer[:layer.index("=")]
                layer_value = layer[layer.index("=") + 1:]
                if layerType == "turkish" or layerType == "english" or layerType == "persian":
                    self.name = layer_value
                    self.__language = AnnotatedWord.getLanguageFromString(layerType)
                elif layerType == "morphologicalAnalysis":
                    self.__parse = MorphologicalParse(layer_value)
                elif layerType == "metaMorphemes":
                    self.__metamorphic_parse = MetamorphicParse(layer_value)
                elif layerType == "namedEntity":
                    self.__named_entity_type = NamedEntityType.getNamedEntityType(layer_value)
                elif layerType == "propbank" or layerType == "propBank":
                    self.__argument_list = ArgumentList(layer_value)
                elif layerType == "framenet" or layerType == "frameNet":
                    self.__frame_element_list = FrameElementList(layer_value)
                elif layerType == "shallowParse":
                    self.__shallow_parse = layer_value
                elif layerType == "semantics":
                    self.__semantic = layer_value
                elif layerType == "slot":
                    self.__slot = Slot(layer_value)
                elif layerType == "polarity":
                    self.setPolarity(layer_value)
                elif layerType == "universalDependency":
                    values = layer_value.split("$")
                    self.__universal_dependency = UniversalDependencyRelation(int(values[0]), values[1])
                elif layerType == "ccg":
                    self.__ccg = layer_value
                elif layerType == "posTag":
                    self.__pos_tag = layer_value
        elif isinstance(layerType, NamedEntityType):
            super().__init__(word)
            self.__named_entity_type = layerType
            self.__argument = Argument("NONE")
        elif isinstance(layerType, MorphologicalParse):
            super().__init__(word)
            self.__parse = layerType
            self.__named_entity_type = NamedEntityType.NONE
            self.__argument = Argument("NONE")
        elif isinstance(layerType, FsmParse):
            super().__init__(word)
            self.__parse = layerType
            self.__named_entity_type = NamedEntityType.NONE
            self.setMetamorphicParse(layerType.withList())
            self.__argument = Argument("NONE")

    def __str__(self) -> str:
        """
        Converts an AnnotatedWord to string. For each annotation layer, the method puts a left brace, layer name,
        equal sign and layer value finishing with right brace.

        RETURNS
        -------
        str
            String form of the AnnotatedWord.
        """
        result = ""
        if self.__language == Language.TURKISH:
            result = "{turkish=" + self.name + "}"
        elif self.__language == Language.ENGLISH:
            result = "{english=" + self.name + "}"
        elif self.__language == Language.PERSIAN:
            result = "{persian=" + self.name + "}"
        if self.__parse is not None:
            result = result + "{morphologicalAnalysis=" + self.__parse.__str__() + "}"
        if self.__metamorphic_parse is not None:
            result = result + "{metaMorphemes=" + self.__metamorphic_parse.__str__() + "}"
        if self.__semantic is not None:
            result = result + "{semantics=" + self.__semantic + "}"
        if self.__named_entity_type is not None:
            result = result + "{namedEntity=" + NamedEntityType.getNamedEntityString(self.__named_entity_type) + "}"
        if self.__argument_list is not None:
            result = result + "{propbank=" + self.__argument_list.__str__() + "}"
        if self.__frame_element_list is not None:
            result = result + "{framenet=" + self.__frame_element_list.__str__() + "}"
        if self.__slot is not None:
            result = result + "{slot=" + self.__slot.__str__() + "}"
        if self.__shallow_parse is not None:
            result = result + "{shallowParse=" + self.__shallow_parse + "}"
        if self.__polarity is not None:
            result = result + "{polarity=" + self.getPolarityString() + "}"
        if self.__universal_dependency is not None:
            result = result + "{universalDependency=" + self.__universal_dependency.to().__str__() + "$" + \
                     self.__universal_dependency.__str__() + "}"
        if self.__ccg is not None:
            result = result + "{ccg=" + self.__ccg + "}"
        if self.__pos_tag is not None:
            result = result + "{posTag=" + self.__pos_tag + "}"
        return result

    def getLayerInfo(self, viewLayerType: ViewLayerType) -> str:
        """
        Returns the value of a given layer.

        PARAMETERS
        ----------
        viewLayerType : ViewLayerType
            Layer for which the value questioned.

        RETURNS
        -------
        str
            The value of the given layer.
        """
        if viewLayerType == ViewLayerType.INFLECTIONAL_GROUP:
            if self.__parse is not None:
                return self.__parse.__str__()
        elif viewLayerType == ViewLayerType.META_MORPHEME:
            if self.__metamorphic_parse is not None:
                return self.__metamorphic_parse.__str__()
        elif viewLayerType == ViewLayerType.SEMANTICS:
            return self.__semantic
        elif viewLayerType == ViewLayerType.NER:
            if self.__named_entity_type is not None:
                return self.__named_entity_type.__str__()
        elif viewLayerType == ViewLayerType.SHALLOW_PARSE:
            return self.__shallow_parse
        elif viewLayerType == ViewLayerType.TURKISH_WORD:
            return self.name
        elif viewLayerType == ViewLayerType.PROPBANK:
            if self.__argument_list is not None:
                return self.__argument_list.__str__()
        elif viewLayerType == ViewLayerType.FRAMENET:
            if self.__frame_element_list is not None:
                return self.__frame_element_list.__str__()
        elif viewLayerType == ViewLayerType.SLOT:
            if self.__slot is not None:
                return self.__slot.__str__()
        elif viewLayerType == ViewLayerType.POLARITY:
            if self.__polarity is not None:
                return self.getPolarityString()
        elif viewLayerType == ViewLayerType.DEPENDENCY:
            if self.__universal_dependency is not None:
                return self.__universal_dependency.to().__str__() + "$" + self.__universal_dependency.__str__()
        elif viewLayerType == ViewLayerType.CCG:
            return self.__ccg
        elif viewLayerType == ViewLayerType.POS_TAG:
            return self.__pos_tag
        else:
            return None

    def getParse(self) -> MorphologicalParse:
        """
        Returns the morphological parse layer of the word.

        RETURNS
        -------
        MorphologicalParse
            The morphological parse of the word.
        """
        return self.__parse

    def setParse(self, parseString: MorphologicalParse):
        """
        Sets the morphological parse layer of the word.

        PARAMETERS
        ----------
        parseString : str
            The new morphological parse of the word in string form.
        """
        if parseString is not None:
            self.__parse = MorphologicalParse(parseString)
        else:
            self.__parse = None

    def getMetamorphicParse(self) -> MetamorphicParse:
        """
        Returns the metamorphic parse layer of the word.

        RETURNS
        -------
        MetamorphicParse
            The metamorphic parse of the word.
        """
        return self.__metamorphic_parse

    def setMetamorphicParse(self, parseString: str):
        """
        Sets the metamorphic parse layer of the word.

        PARAMETERS
        ----------
        parseString : str
            The new metamorphic parse of the word in string form.
        """
        self.__metamorphic_parse = MetamorphicParse(parseString)

    def getSemantic(self) -> str:
        """
        Returns the semantic layer of the word.

        RETURNS
        -------
        str
            Sense id of the word.
        """
        return self.__semantic

    def setSemantic(self, semantic: str):
        """
        Sets the semantic layer of the word.

        PARAMETERS
        ----------
        semantic : str
            New sense id of the word.
        """
        self.__semantic = semantic

    def getNamedEntityType(self) -> NamedEntityType:
        """
        Returns the named entity layer of the word.

        RETURNS
        -------
        NamedEntityType
            Named entity tag of the word.
        """
        return self.__named_entity_type

    def setNamedEntityType(self, namedEntity: str):
        """
        Sets the named entity layer of the word.

        PARAMETERS
        ----------
        namedEntity : str
            New named entity tag of the word.
        """
        if namedEntity is not None:
            self.__named_entity_type = NamedEntityType.getNamedEntityType(namedEntity)
        else:
            self.__named_entity_type = None

    def getArgumentList(self) -> ArgumentList:
        """
        Returns the semantic role layer of the word.

        RETURNS
        -------
        ArgumentList
            Semantic role tag of the word.
        """
        return self.__argument_list

    def setArgumentList(self, argumentList: str):
        """
        Sets the semantic role layer of the word.

        PARAMETERS
        ----------
        argumentList : Argument
            New semantic role tag of the word.
        """
        if argumentList is not None:
            self.__argument = ArgumentList(argumentList)
        else:
            self.__argument = None

    def getFrameElementList(self) -> FrameElementList:
        """
        Returns the framenet layer of the word.

        RETURNS
        -------
        FrameElementList
            Framenet tag of the word.
        """
        return self.__frame_element_list

    def setFrameElementList(self, frameElementList: str):
        """
        Sets the framenet layer of the word.

        PARAMETERS
        ----------
        frameElementList : str
            New framenet tag of the word.
        """
        if frameElementList is not None:
            self.__frame_element_list = FrameElementList(frameElementList)
        else:
            self.__frame_element_list = None

    def getSlot(self) -> Slot:
        """
        Returns the slot layer of the word.

        RETURNS
        -------
        Slot
            Slot tag of the word.
        """
        return self.__slot

    def setSlot(self, slot: str):
        """
        Sets the slot layer of the word.

        PARAMETERS
        ----------
        slot : str
            New slot tag of the word.
        """
        if slot is not None:
            self.__slot = Slot(slot)
        else:
            self.__slot = None

    def getPolarity(self) -> PolarityType:
        """
        Returns the polarity layer of the word.

        RETURNS
        -------
        PolarityType
            Polarity tag of the word.
        """
        return self.__polarity

    def getPolarityString(self) -> str:
        """
        Returns the polarity layer of the word.

        RETURNS
        -------
        str
            Polarity string of the word.
        """
        if self.__polarity == PolarityType.POSITIVE:
            return "positive"
        elif self.__polarity == PolarityType.NEGATIVE:
            return "negative"
        elif self.__polarity == PolarityType.NEUTRAL:
            return "neutral"
        else:
            return "neutral"

    def setPolarity(self, polarity: str):
        """
        Sets the polarity layer of the word.

        PARAMETERS
        ----------
        polarity : str
            New polarity tag of the word.
        """
        if polarity is not None:
            if polarity == "positive" or polarity == "pos":
                self.__polarity = PolarityType.POSITIVE
            elif polarity == "negative" or polarity == "neg":
                self.__polarity = PolarityType.NEGATIVE
            else:
                self.__polarity = PolarityType.NEUTRAL
        else:
            self.__polarity = None

    def getShallowParse(self) -> str:
        """
        Returns the shallow parse layer of the word.

        RETURNS
        -------
        str
            Shallow parse tag of the word.
        """
        return self.__shallow_parse

    def setShallowParse(self, parse: str):
        """
        Sets the shallow parse layer of the word.

        PARAMETERS
        ----------
        parse : str
            New shallow parse tag of the word.
        """
        self.__shallow_parse = parse

    def getCcg(self) -> str:
        """
        Returns the ccg layer of the word.

        RETURNS
        -------
        str
            Ccg tag of the word.
        """
        return self.__ccg

    def setCcg(self, ccg: str):
        """
        Sets the ccg layer of the word.

        PARAMETERS
        ----------
        ccg : str
            New ccg tag of the word.
        """
        self.__ccg = ccg

    def getPosTag(self) -> str:
        """
        Returns the posTag layer of the word.

        RETURNS
        -------
        str
            Pos tag of the word.
        """
        return self.__pos_tag

    def setPosTag(self, posTag: str):
        """
        Sets the posTag layer of the word.

        PARAMETERS
        ----------
        posTag : str
            New pos tag of the word.
        """
        self.__pos_tag = posTag

    def getUniversalDependency(self) -> UniversalDependencyRelation:
        """
        Returns the universal dependency layer of the word.

        RETURNS
        -------
        UniversalDependencyRelation
            Universal dependency relation of the word.
        """
        return self.__universal_dependency

    def setUniversalDependency(self, to: int, dependencyType: str):
        """
        Sets the universal dependency layer of the word.

        PARAMETERS
        ----------
        to : int
            to Word related to.
        dependencyType : str
            type of dependency the word is related to.
        """
        if to < 0:
            self.__universal_dependency = None
        else:
            self.__universal_dependency = UniversalDependencyRelation(to, dependencyType)

    def getUniversalDependencyFormat(self, sentenceLength: int) -> str:
        """
        Returns the connlu format string for this word. Adds surface form, root, universal pos tag, features, and
        universal dependency information.
        :param sentenceLength: Number of words in the sentence.
        :return: The connlu format string for this word.
        """
        if self.__parse is not None:
            uPos = self.__parse.getUniversalDependencyPos()
            result = self.name + "\t" + self.__parse.getWord().getName() + "\t" + \
                     uPos + "\t_\t"
            features = self.__parse.getUniversalDependencyFeatures(uPos)
            if len(features) == 0:
                result = result + "_"
            else:
                first = True
                for feature in features:
                    if first:
                        first = False
                    else:
                        result += "|"
                    result += feature
            result += "\t"
            if self.__universal_dependency is not None and self.__universal_dependency.to() <= sentenceLength:
                result += self.__universal_dependency.to().__str__() + "\t" + \
                          self.__universal_dependency.__str__().lower() + "\t"
            else:
                result += "_\t_\t"
            result += "_\t_"
            return result
        else:
            return self.name + "\t" + self.name + "\t_\t_\t_\t_\t_\t_\t_"

    def getFormattedString(self, wordFormat: WordFormat):
        if wordFormat == WordFormat.SURFACE:
            return self.name
        return self.name

    def checkGazetteer(self, gazetteer: Gazetteer):
        """
        Checks the gazetteer and sets the named entity tag accordingly.
        :param gazetteer: Gazetteer used to set named entity tag.
        """
        word_lower_case = self.name.lower()
        if gazetteer.contains(word_lower_case) and self.__parse.containsTag(MorphologicalTag.PROPERNOUN):
            self.setNamedEntityType(gazetteer.getName())
        if "'" in word_lower_case and gazetteer.contains(word_lower_case[:word_lower_case.index("'")]) and \
                self.__parse.containsTag(MorphologicalTag.PROPERNOUN):
            self.setNamedEntityType(gazetteer.getName())

    def getLanguage(self) -> Language:
        """
        Returns the language of the word.

        RETURNS
        ----------
        The language of the word.
        """
        return self.__language

    @staticmethod
    def getLanguageFromString(languageString: str) -> Language:
        """
        Converts a language string to language.

        PARAMETERS
        ----------
        languageString : str
            String defining the language name.

        RETURNS
        ----------
        Language corresponding to the languageString.
        """
        if languageString == "turkish" or languageString == "Turkish":
            return Language.TURKISH
        elif languageString == "english" or languageString == "English":
            return Language.ENGLISH
        elif languageString == "persian" or languageString == "Persian":
            return Language.PERSIAN
        else:
            return Language.TURKISH



from __future__ import annotations

import os
import re

from Corpus.Corpus import Corpus

from AnnotatedSentence.AnnotatedSentence import AnnotatedSentence
from AnnotatedSentence.AnnotatedWord import AnnotatedWord
from DependencyParser.ParserEvaluationScore import ParserEvaluationScore


class AnnotatedCorpus(Corpus):

    def __init__(self,
                 folder: str,
                 pattern: str = None):
        """
        A constructor of AnnotatedCorpus class which reads all AnnotatedSentence files with the file
        name satisfying the given pattern inside the given folder. For each file inside that folder, the constructor
        creates an AnnotatedSentence and puts in inside the list parseTrees.

        PARAMETERS
        ----------
        folder : str
            Folder where all sentences reside.
        pattern : str
            File pattern such as "." ".train" ".test".
        """
        self.sentences = []
        for root, dirs, files in os.walk(folder):
            for file in files:
                file_name = os.path.join(root, file)
                if (pattern is None or pattern in file_name) and re.match("\\d+\\.", file):
                    f = open(file_name, "r", encoding='utf8')
                    sentence = AnnotatedSentence(f, file_name)
                    self.sentences.append(sentence)

    def compareParses(self, corpus: AnnotatedCorpus) -> ParserEvaluationScore:
        """
        Compares the corpus with the given corpus and returns a parser evaluation score for this comparison. The result
        is calculated by summing up the parser evaluation scores of sentence by sentence dependency relation comparisons.
        :param corpus: Corpus to be compared.
        :return: A parser evaluation score object.
        """
        result = ParserEvaluationScore()
        for i in range(len(self.sentences)):
            sentence1 = self.sentences[i]
            sentence2 = corpus.getSentence(i)
            result.add(sentence1.compareParses(sentence2))
        return result

    def exportUniversalDependencyFormat(self,
                                        outputFileName: str,
                                        path: str = None):
        """
        Exports the annotated corpus as a UD file in connlu format. Every sentence is converted into connlu format and
        appended to the output file. Multiple paths are possible in the annotated corpus. This method outputs the
        sentences in the given path.
        :param outputFileName: Output file name in connlu format.
        :param path: Current path for the part of the annotated corpus.
        """
        file = open(outputFileName, "w")
        for i in range(self.sentenceCount()):
            sentence = self.getSentence(i)
            if isinstance(sentence, AnnotatedSentence):
                file.write(sentence.getUniversalDependencyFormat(path))
        file.close()

    def checkMorphologicalAnalysis(self):
        """
        The method traverses all words in all sentences and prints the words which do not have a morphological analysis.
        """
        for i in range(self.sentenceCount()):
            sentence = self.getSentence(i)
            if isinstance(sentence, AnnotatedSentence):
                for j in range(sentence.wordCount()):
                    word = sentence.getWord(j)
                    if isinstance(word, AnnotatedWord):
                        if word.getParse() is None:
                            print("Morphological Analysis does not exist for sentence " + sentence.getFileName())
                            break

    def checkNer(self):
        """
        The method traverses all words in all sentences and prints the words which do not have named entity annotation.
        """
        for i in range(self.sentenceCount()):
            sentence = self.getSentence(i)
            if isinstance(sentence, AnnotatedSentence):
                for j in range(sentence.wordCount()):
                    word = sentence.getWord(j)
                    if isinstance(word, AnnotatedWord):
                        if word.getNamedEntityType() is None:
                            print("NER annotation does not exist for sentence " + sentence.getFileName())
                            break

    def checkShallowParse(self):
        """
        The method traverses all words in all sentences and prints the words which do not have shallow parse annotation.
        """
        for i in range(self.sentenceCount()):
            sentence = self.getSentence(i)
            if isinstance(sentence, AnnotatedSentence):
                for j in range(sentence.wordCount()):
                    word = sentence.getWord(j)
                    if isinstance(word, AnnotatedWord):
                        if word.getShallowParse() is None:
                            print("Shallow parse annotation does not exist for sentence " + sentence.getFileName())
                            break

    def checkSemantic(self):
        """
        The method traverses all words in all sentences and prints the words which do not have sense annotation.
        """
        for i in range(self.sentenceCount()):
            sentence = self.getSentence(i)
            if isinstance(sentence, AnnotatedSentence):
                for j in range(sentence.wordCount()):
                    word = sentence.getWord(j)
                    if isinstance(word, AnnotatedWord):
                        if word.getSemantic() is None:
                            print("Semantic annotation does not exist for sentence " + sentence.getFileName())
                            break



from setuptools import setup

from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text(encoding="utf-8")

setup(
    name='NlpToolkit-AnnotatedSentence',
    version='1.0.39',
    packages=['AnnotatedSentence'],
    url='https://github.com/olcaytaner/AnnotatedSentence-Py',
    license='',
    author='olcaytaner',
    author_email='olcaytaner@isikun.edu.tr',
    description='Annotated Sentence Processing Library',
    install_requires=['NlpToolkit-WordNet', 'NlpToolkit-NamedEntityRecognition', 'NlpToolkit-PropBank', 'NlpToolkit-DependencyParser', 'NlpToolkit-FrameNet', 'NlpToolkit-SentiNet'],
    long_description=long_description,
    long_description_content_type='text/markdown'
)
